# -*- coding: utf-8 -*-

"""
Author: Zhou Zhi-Jian
Institution: Hunan University
Email: zjzhou@hnu.edu.cn
Copyright：Copyright (c) Zhou Zhi-Jian
Time: 2022/4/17 21:32

"""
import sys
import os
import numpy as np
import pandas as pd
import scipy.stats as stats
import matplotlib
from multiprocessing import Pool
from sequence_align import SeqAlign

matplotlib.use("agg")  # avoid the  ModuleNotFoundError: No module named '_tkinter'

import matplotlib.pyplot as plt

from plt_corlor_list import plt_corlor



def handle_exceptions(parameter_dic):


    if parameter_dic["out_dir"].strip() == "":
        print(
            ">>> Error！Please specify an output directory by '-o' options." + "\n")

        return True


    elif (parameter_dic["unaligned_seq"] == ""
          and parameter_dic["aligned_seq"] == ""
          and parameter_dic["input_wic"] == ""):

        print(">>> Error！There is no input file." + "\n")

        return True


    elif parameter_dic["query_lineage_name"] == "":
        print(">>> Error！Please specify the query lineage by '-q' options." + "\n")

        return True


    elif parameter_dic["gaps_use"] == "":
        print(
            ">>> Error！Please specify the value of '-g' options." + "\n")

        return True




def handle_input_file(parameter_dic):

    if parameter_dic["aligned_seq"] != "":

        seq_alignment_file = parameter_dic["aligned_seq"].replace("\\", "/")

        return seq_alignment_file


    elif parameter_dic["unaligned_seq"] != "":

        if parameter_dic["align_tool"] != "":

            align_record = parameter_dic["out_dir"] + "/" + "align_record"

            make_dir(align_record)

            input_seq_dir, input_prefix = resolve_file_path(parameter_dic["unaligned_seq"])

            seq_alignment_file = (align_record + "/" + input_prefix
                                  + "_" + parameter_dic["align_tool"]
                                  + ".fasta")

            SeqAlign(parameter_dic["align_tool"],
                     parameter_dic["thread_num"],
                     parameter_dic["unaligned_seq"],
                     seq_alignment_file)

            file_size = os.path.getsize(seq_alignment_file)

            if os.path.exists(seq_alignment_file) and file_size > 0:

                file_size = os.path.getsize(seq_alignment_file)

                if file_size > 0:

                    print(">>> Sequence alignment has been completed!" + "\n")

                    return seq_alignment_file


            else:

                print(">>> Error！Multiple sequence alignments did not succeed." + "\n")

                sys.exit()


        else:
            print(">>> Error！Please specify a tool used for sequence alignment, "
                  "such as '-at mafft', or '-at muscle', or '-at clustalo'." + "\n")

            sys.exit()



    else:
        sys.exit()



def get_all_path(open_dir_path):

    rootdir = open_dir_path

    path_list = []

    lists = os.listdir(rootdir)

    for i in range(0, len(lists)):
        com_path = os.path.join(rootdir, lists[i])
        com_path = com_path.replace("\\", "/")
        # print(com_path)

        if os.path.isfile(com_path):
            path_list.append(com_path)


    return path_list



def make_dir(input_dir):
    """
    :param input_dir: directory which need to create
    :return:
    """
    if not os.path.isdir(input_dir):  #  = False
        os.makedirs(input_dir)


def resolve_file_path(file_path):

    file_path = file_path.replace("\\", "/")

    input_data_dir = os.path.dirname(file_path)

    inputfile_name = file_path.split("/")[-1]

    file_farmat = inputfile_name.split(".")[-1]

    out_prefix = inputfile_name.replace(file_farmat,"").rstrip(".")

    return input_data_dir, out_prefix




def mark_linegaes(lineage_file_dir, outdir):

    print(">>> VirusRecom is running..." + "\n")

    lineage_file_list = get_all_path(lineage_file_dir)

    out_file = outdir.replace("\\", "/") + "/merge_with_mark.fasta"

    with open(out_file,"w",encoding="utf-8") as out_put:

        for each_path in lineage_file_list:
            # print(each_path)

            each_path = each_path.replace("\\", "/")

            input_data_dir, out_prefix = resolve_file_path(each_path)

            with open(each_path, "r", encoding="utf-8") as lineage_file_input:
                out_put.write(
                    lineage_file_input.read().replace(">",
                                                ">" + out_prefix + "_")+ "\n")

        print(">>> Finished, the output file is "
              + "'" + out_file + "'"
              + "\n")



def export_seqname(seq_file, outdir):

    print(">>> VirusRecom is running..." + "\n")

    seq_file = seq_file.replace("\\", "/")

    outdir = outdir.replace("\\", "/")

    out_file = outdir + "/" + "sequence_name.txt"

    with open(out_file, "w", encoding="utf-8") as output_file:

        with open(seq_file, "r", encoding="utf-8") as input_file:

            for line in input_file:
                line = line.strip()
                if line.startswith(">"):
                    output_file.write(line.replace(">", "") + "\n")

        print(">>> Finished, the output file is "
              + "'" + out_file + "'"
              + "\n")


def others_analysis(parameter_dic):

    if parameter_dic["engrave_name"] != "":

        mark_linegaes(parameter_dic["engrave_name"],
                      parameter_dic["out_dir"])

        return True

    elif parameter_dic["export_name"] != "":
        export_seqname(parameter_dic["export_name"],
                       parameter_dic["out_dir"])

        return True

    else:
        return False


# def read_seq(file_path):
#
#     seq_tab = []
#
#     with open(file_path, "r", encoding="utf-8") as gen_file_input:
#
#         gen_file_list = gen_file_input.read().split(">")
#
#         for n in range(1, int(len(gen_file_list))):
#             seq = gen_file_list[n].strip()
#             st = []
#             seq_name = seq.split("\n")[0]
#             # print(seq_name)
#
#             seq_contain = seq.replace("\n", "").replace(seq_name, "").upper()
#
#             st.append(seq_name)
#
#
#             st += list(seq_contain.strip())
#
#             seq_tab.append(st)
#
#
#         seq_sites_list = []
#         seq_sites_list.append("SeqName")
#
#         seq_length = len(seq_tab[0]) -1
#
#         for x in range(seq_length):
#             seq_sites_list.append(str(x + 1))
#
#         seq_pands = pd.DataFrame(seq_tab,columns=seq_sites_list).set_index("SeqName")
#
#
#         return seq_pands



def read_seq(input_file_path):

    with open(input_file_path, "r",encoding="utf-8") as gen_file:

        seq_tab = []

        seq_dic = {}

        seq_name_list = []

        seq_name = ""

        for line in gen_file:
            line = line.strip()

            if line.startswith(">"):
                seq_name = line.strip(">")
                seq_name_list.append(seq_name)
                seq_dic[seq_name] = []

            elif line != "":
                seq_dic[seq_name].append(line)


        for each_seqname in seq_name_list:

            seq = "".join(seq_dic[each_seqname]).upper()

            seq_tab.append([each_seqname] + list(seq))

        seq_sites_list = ["SeqName"]

        seq_length = len(seq_tab[0]) - 1

        for k in range(seq_length):
            seq_sites_list.append(str(k + 1))

        seq_pands = pd.DataFrame(seq_tab, columns=seq_sites_list).set_index("SeqName")

        return seq_pands




def calEnt(siteData, mode):

    # print(siteData)

    if mode == "single":
        return 2

    else:
        n = siteData.shape[0]
        nt_count = siteData.value_counts()
        # print(nt_count)

        p = nt_count / n
        # print(p)

        ent = (-p * np.log2(p)).sum()

        IC = 2 - ent

        return IC



def calEnt_gap(siteData, mode):

    # print(siteData)

    if mode == "single":
        return np.log2(5)

    else:

        n = siteData.shape[0]
        nt_count = siteData.value_counts()
        # print(nt_count)

        p = nt_count / n
        # print(p)

        ent = (-p * np.log2(p)).sum()

        IC = np.log2(5) - ent

        return IC




def lineage_mwic_run(lineage_name,
                     slither_window_list,
                     lineage_wic_df):

    each_lineage_setp_pro = []


    for each_window in slither_window_list:
        start_row = each_window[0]

        end_row = each_window[1]

        step_df = list(lineage_wic_df[start_row:end_row])

        mean_ic_per_win = np.mean(step_df)

        each_lineage_setp_pro.append(mean_ic_per_win)


    return [lineage_name,each_lineage_setp_pro]




def mwic_calculation(sites_probability_data,
                     lineage_name_list,
                     sites_count,site_map_dic,
                     windows_size,step_size,
                     output_path,thread_num):

    step_probability_data = pd.DataFrame()

    slid_number = int(sites_count / step_size)

    window_center_index = []
    window_center_original = []

    slither_window_list = []


    for n in range(slid_number):

        start_row = step_size * n

        end_row = min(start_row + windows_size, sites_count - 1)

        center_index = int((start_row + end_row) / 2)

        slither_window_list.append([start_row, end_row, center_index])

        center_site = center_index + 1

        window_center_index.append(center_site)

        _original_site = int(site_map_dic[str(center_site)])

        window_center_original.append(_original_site)

        if end_row == sites_count - 1:
            break


    step_probability_data["Central_position(original)"] = window_center_original

    step_probability_data["Central_position(current)"] = window_center_index


    p = Pool(int(thread_num))

    lineage_mwic_list = []


    for each_lineage in lineage_name_list:

        lineage_wic_df = sites_probability_data[each_lineage]

        mwic_result = p.apply_async(lineage_mwic_run,
                                       args=(each_lineage,slither_window_list,lineage_wic_df))

        lineage_mwic_list.append(mwic_result)

    p.close()
    p.join()

    for each_calculation in lineage_mwic_list:
        lineage_mwic = each_calculation.get()  # get()方法获取计算结果

        step_probability_data[lineage_mwic[0]] = lineage_mwic[1]


    # step_probability_data.to_excel(
    #     excel_writer=output_path,
    #     index=False)

    step_probability_data.to_csv(output_path, index=False, sep=",")

    return step_probability_data,slither_window_list




def recomsplicing(sites_probability_data,
                  sites_count,
                  lineage_name_list,
                  recombination_frag,
                  step_size,max_mic,
                  max_recom_fragment,
                  recom_percentage):

    recom_region_dic = {}

    for each_lineage in recombination_frag:
        lineage_frag_list = recombination_frag[each_lineage]

        if lineage_frag_list != []:

            frag_count = len(lineage_frag_list)
            # print(frag_count)

            cursor_site = 1
            cursor_center = lineage_frag_list[cursor_site - 1]

            detected_area = []

            while cursor_site <= frag_count:

                flage_label = False

                breakpoint_judgment = []

                for i in range(cursor_site - 1, frag_count):

                    region_left = max(0, int(
                        cursor_center - step_size / 2))
                    region_right = min(sites_count, int(
                        lineage_frag_list[i] + step_size / 2))

                    region_sites_count = region_right - region_left + 1

                    lineage_Ri_wic = sum(list(
                        sites_probability_data[each_lineage][
                        region_left:region_right]))

                    max_lineage_ic = 0

                    for lineage in lineage_name_list:
                        if lineage != each_lineage:
                            lineage_ic = sum(list(
                                sites_probability_data[lineage][
                                region_left:region_right]))
                            if lineage_ic >= max_lineage_ic:
                                max_lineage_ic = lineage_ic

                                # print(max_lineage_ic)

                    if (lineage_Ri_wic > max_lineage_ic
                            and lineage_Ri_wic / (
                                    region_sites_count * max_mic) >= recom_percentage):
                        breakpoint_judgment.append([i + 1, region_left,
                                                    region_right,
                                                    "True"])

                        flage_label = True

                if cursor_site == frag_count:
                    break

                if flage_label == False:

                    cursor_site = cursor_site + 1

                    cursor_center = lineage_frag_list[cursor_site - 1]

                else:

                    first_each_region = breakpoint_judgment[0]
                    first_ri = first_each_region[2] - first_each_region[1] + 1

                    if first_ri > max_recom_fragment:
                        cursor_site = cursor_site + 1
                        cursor_center = lineage_frag_list[cursor_site - 1]

                    else:

                        max_Ri = breakpoint_judgment[-1]

                        max_Ri_sites = max_Ri[2] - max_Ri[1] + 1

                        if max_Ri_sites <= max_recom_fragment:
                            detected_area.append([max_Ri[1], max_Ri[2]])

                            break

                        else:

                            for n in range(len(breakpoint_judgment)):
                                each_region = breakpoint_judgment[n]
                                ri = each_region[2] - each_region[
                                    1] + 1

                                if ri > max_recom_fragment:
                                    local_max_Ri = breakpoint_judgment[n - 1]
                                    detected_area.append([local_max_Ri[1],
                                                          local_max_Ri[2]])

                                    last_breakpoint = breakpoint_judgment[n - 1][0]

                                    if last_breakpoint == cursor_site:
                                        cursor_site = cursor_site + 1
                                        cursor_center = lineage_frag_list[
                                            cursor_site - 1]

                                    else:
                                        cursor_site = last_breakpoint
                                        cursor_center = lineage_frag_list[
                                            cursor_site - 1]

                                    break

            recom_region_dic[each_lineage] = detected_area


    for i in list(recom_region_dic.keys()):
        if not recom_region_dic[i]:  #  == []
            del recom_region_dic[i]


    return recom_region_dic




def wic_plot(lineage_name,
             site_list,
             input_data,
             query_lineage_name,
             output_path):


    lineage_count = len(lineage_name)

    fig_high = int(lineage_count * 2)

    fig, ax = plt.subplots(len(lineage_name), 1,
                           figsize=(
                           lineage_count, fig_high))

    fig.suptitle("Query seq: " + query_lineage_name, family="Arial")
    fig.tight_layout()

    cm1 = plt.cm.get_cmap("Reds")

    plt.subplots_adjust(top=0.95)

    for n in range(lineage_count):
        each_lineage = lineage_name[n]
        ax_n = ax[n]
        y = list(input_data[each_lineage])
        xx = ax_n.scatter(site_list, y, c=y,
                          label=each_lineage, s=5,
                          cmap=cm1)

        fig.colorbar(xx, ax=ax_n)

        ax_n.set_ylabel("WIC", family="Arial")

        ax_n.legend(loc="best")

        x1_label = ax_n.get_xticklabels()
        [x1_label_temp.set_fontname("Arial") for x1_label_temp in
         x1_label]
        y1_label = ax_n.get_yticklabels()
        [y1_label_temp.set_fontname("Arial") for y1_label_temp in
         y1_label]


    plt.xlabel("Site in alignment", family="Arial")
    plt.savefig(output_path)
    plt.clf()
    plt.close()



def mwic_plot(gap_used,
              lineage_name_list,
              window_center_original,
              step_probability_data,
              query_seq_prefix,
              output_file,
              y_start,
              legend_location):


    fig, ax = plt.subplots()
    ax.spines["right"].set_visible(False)
    ax.spines["top"].set_visible(False)

    if gap_used.upper() == "N":

        plt.ylim((y_start, 2))

    else:
        plt.ylim((y_start, 2.5))

    for n in range(len(lineage_name_list)):
        each_lineage = lineage_name_list[n]

        try:
            plt.plot(window_center_original,
                     list(step_probability_data[each_lineage]),
                     label=each_lineage, color=plt_corlor[n], linewidth=1)
        except:
            plt.plot(window_center_original,
                     list(step_probability_data[each_lineage]),
                     label=each_lineage, color="black", linewidth=1)
        finally:
            pass


    legend_font_size = 8

    if len(lineage_name_list) >= 15:
        legend_font_size = 6

    elif len(lineage_name_list) >= 25:
        legend_font_size = 4


    legend_font = {"family": "Arial",
             "weight": "normal",
             "size": legend_font_size}


    if legend_location.upper() == "R":

        num1 = 1.05
        num2 = 1
        num3 = 2
        num4 = 0
        plt.legend(bbox_to_anchor=(num1, num2), loc=num3, borderaxespad=num4,prop=legend_font)
        plt.subplots_adjust(bottom=0.10, right=0.7)


    else:
        plt.legend(prop=legend_font)

        plt.subplots_adjust(bottom=0.10)


    plt.margins(0)
    # plt.subplots_adjust(bottom=0.10)
    plt.xlabel("Site in alignment", family="Arial")
    plt.ylabel("Mean of weighted information content", family="Arial")
    plt.title("Query sequence: " + query_seq_prefix, family="Arial")


    x1_label = ax.get_xticklabels()
    [x1_label_temp.set_fontname("Arial") for x1_label_temp in
     x1_label]
    y1_label = ax.get_yticklabels()
    [y1_label_temp.set_fontname("Arial") for y1_label_temp in
     y1_label]

    # plt.show()
    plt.savefig(output_file)

    plt.clf()

    plt.close()




def recombreak_run(lineage,lineage_wic,
                   run_number,breakwins):

    negative_lg_p_list = []


    for i in range(run_number):
        start_site = i

        end_site = i + breakwins

        central_pos = int((start_site + end_site) / 2)

        left_region = list(lineage_wic[start_site: central_pos - 1])
        right_region = list(lineage_wic[central_pos + 1: end_site])

        try:

            zihe_test = stats.mannwhitneyu(left_region, right_region,
                                           alternative="two-sided")
            p_value = zihe_test[1]
            negative_lg_p = - np.log10(p_value)
            negative_lg_p_list.append(negative_lg_p)

        except:
            negative_lg_p = 0
            negative_lg_p_list.append(negative_lg_p)

        finally:
            pass



    return [lineage,negative_lg_p_list]



def recombreak_plot(sites_probability_data,
                    lineage_name_list,
                    sites_count,
                    breakwins,
                    site_map_dic,
                    site_dir,
                    query_seq_prefix,
                    thread_num):


    break_p_map = (site_dir + "/"
                   + query_seq_prefix
                   + "_ -lg(p-value)_for_potential_breakpoint.pdf")

    break_p_data = (site_dir + "/"
                    + query_seq_prefix
                    + "_ -lg(p-value)_for_potential_breakpoint.xlsx")

    breakpoint_data = pd.DataFrame()

    run_number = sites_count - breakwins + 1

    central_pos_list = []

    lineage_count = len(lineage_name_list)

    for k in range(run_number):
        start_site = k

        end_site = k + breakwins

        central_pos = int((start_site + end_site) / 2)

        original_site = int(site_map_dic[str(central_pos + 1)])

        central_pos_list.append(original_site)

    breakpoint_data["Original_Index"] = central_pos_list

    p2 = Pool(int(thread_num))

    ca_result_list = []

    for lineage in lineage_name_list:

        lineage_wic = sites_probability_data[lineage]

        ca_result = p2.apply_async(recombreak_run,
                                    args=(lineage, lineage_wic,
                                          run_number,breakwins))

        ca_result_list.append(ca_result)

    p2.close()
    p2.join()


    for each_calculation in ca_result_list:
        lineage_ca = each_calculation.get()

        breakpoint_data[lineage_ca[0]] = lineage_ca[1]

    # print(breakpoint_data)

    breakpoint_data.to_excel(
        excel_writer=break_p_data,
        index=False)



    fig_high2 = int(lineage_count * 2)

    figs, axs = plt.subplots(lineage_count, 1, figsize=(lineage_count,
                                                        fig_high2))

    figs.suptitle("Query seq: " + query_seq_prefix, family="Arial")
    figs.tight_layout()

    plt.subplots_adjust(top=0.95)

    for n in range(lineage_count):
        each_lineage = lineage_name_list[n]
        ax_n = axs[n]
        y = list(breakpoint_data[each_lineage])

        try:

            ax_n.plot(central_pos_list, y,
                      label=query_seq_prefix + " : " + each_lineage,
                      color=plt_corlor[n], )

        except:
            ax_n.plot(central_pos_list, y,
                      label=query_seq_prefix + " : " + each_lineage,
                      color="black", )

        finally:
            pass

        ax_n.set_ylabel("-lg(P)", family="Arial")

        ax_n.legend(loc="best")

        x1_label = ax_n.get_xticklabels()
        [x1_label_temp.set_fontname("Arial") for x1_label_temp in
         x1_label]
        y1_label = ax_n.get_yticklabels()
        [y1_label_temp.set_fontname("Arial") for y1_label_temp in
         y1_label]

    plt.xlabel("Site in alignment", family="Arial")

    plt.savefig(break_p_map)

    plt.clf()

    plt.close()








# -*- coding: utf-8 -*-

"""
Author: Zhou Zhi-Jian
Email: zjzhou@hnu.edu.cn
Time: 2024/7/22 0:40

"""


import sys
import os
import psutil
import pandas as pd

from multiprocessing import Pool



def calculate_memory():
    pid = os.getpid()
    p = psutil.Process(pid)
    info = p.memory_full_info()
    memory = info.uss / 1024 / 1024
    return memory


def gap_processing(seq_df,gaps_use):

    gap_site = []

    if gaps_use.upper() == "N":

        print("    " + "VirusRecom is removing sites (columns) containing gap (-)..." + "\n")

        old_site_label = list(seq_df.columns)

        seq_pd_clean = seq_df[~seq_df.isin(["-"])].dropna(axis=1,how='any')  # 去除含有gap的列（位点)

        new_site_label = list(seq_pd_clean.columns)

        for each_site in old_site_label:
            if each_site not in new_site_label:
                gap_site.append(each_site)


        return seq_pd_clean,gap_site

    else:
        return seq_df,gap_site


def site_model(seq_df, method):

    invariant_site = []

    if method.upper() == "P":

        print("    " + "VirusRecom is extracting polymorphic sites..." + "\n")

        old_site_label = list(seq_df.columns)

        seq_pd_clean = seq_df.loc[:, (seq_df != seq_df.iloc[0]).any()]

        new_site_label = list(seq_pd_clean.columns)

        for each_site in old_site_label:
            if each_site not in new_site_label:
                invariant_site.append(each_site)

        return seq_pd_clean, invariant_site

    else:
        return seq_df, invariant_site




def piecewise_read(seq_file_path,
                   block_size,
                   data_blocks):

    print("    " + "VirusRecom is importing data blocks {}".format(data_blocks) + "\n")

    with open(seq_file_path, "r",encoding="utf-8") as gen_file:

        seq_tab = []

        seq_dic = {}

        seq_name_list = []

        seq_name = ""

        for line in gen_file:
            line = line.strip()

            if line.startswith(">"):
                seq_name = line.strip(">")
                seq_name_list.append(seq_name)
                seq_dic[seq_name] = []

            elif line != "":
                seq_dic[seq_name].append(line)


        for each_seqname in seq_name_list:

            seq = "".join(seq_dic[each_seqname]).upper()

            seq_tab.append([each_seqname] + list(seq))

        seq_sites_list = ["SeqName"]

        seq_length = len(seq_tab[0]) - 1

        for k in range(seq_length):
            seq_sites_list.append(str(k + 1))

        seq_pands = pd.DataFrame(seq_tab, columns=seq_sites_list).set_index("SeqName")

        site_start = block_size * (data_blocks - 1)  #

        site_end = block_size * data_blocks


        if site_end >= seq_length:

            print("    Load sites: " + "{}".format(str(site_start + 1))
                  + " - {}".format(seq_length) + "\n")

            if data_blocks == 1:

                return seq_pands, False

            else:

                sub_pands = seq_pands.iloc[:, site_start:seq_length]

                return sub_pands,False

        else:
            sub_pands = seq_pands.iloc[:, site_start:site_end]

            print("    Load sites: " + "{}".format(str(site_start + 1))
                                  + " - {}".format(site_end) + "\n")

            return sub_pands,True





def lineage_wic_run(query_seq, each_lineage,
                    lineage_seq_df,used_calEnt,
                    query_seq_prefix):

    ent_probability = []

    query_seq_count = query_seq.shape[0]

    for (columnName, columnData) in lineage_seq_df.items():

        lineage_site = list(columnData)

        if len(lineage_site) == 1:
            IC = used_calEnt(columnData, "single")

        else:

            IC = used_calEnt(columnData, "")

        if query_seq_count == 1:
            query_nt = query_seq[columnName][0]
            query_nt_ratio = 1

            try:

                query_nt_lineage_ratio = lineage_site.count(query_nt) / \
                                         columnData.shape[0]

                p_ent = query_nt_lineage_ratio * IC * query_nt_ratio

                ent_probability.append(p_ent)


            except:

                print(">>> Error! The lineage '" + each_lineage + "'"
                      + " is not in sequence data." + "\n")

                sys.exit()


        else:

            query_seq_site_list = list(query_seq[columnName])

            query_seq_site_count = len(query_seq_site_list)

            maxpro_nt = max(query_seq_site_list,
                            key=query_seq_site_list.count)

            query_nt_ratio = query_seq_site_list.count(
                maxpro_nt) / query_seq_site_count

            try:

                query_nt_lineage_ratio = lineage_site.count(maxpro_nt) / \
                                         columnData.shape[0]

                p_ent = query_nt_lineage_ratio * IC * query_nt_ratio

                ent_probability.append(p_ent)


            except:

                print(">>> Error! The lineage '" + each_lineage + "'"
                      + " is not in sequence data." + "\n")

                sys.exit()


    # print("    " +
    #       "The calculation of " + each_lineage + "'s recombination contribution to "
    #       + query_seq_prefix + " has been completed!" + "\n")

    return [each_lineage, ent_probability]



def wic_calculation(seq_data,
                    lineage_name_list,
                    query_seq_prefix,
                    parameter_dic,
                    site_idenx):


    used_calEnt = parameter_dic["calEnt_use"]

    thread_num = parameter_dic["thread_num"]


    query_seq = seq_data[seq_data.index.str.contains(query_seq_prefix) == True]

    query_seq_count = query_seq.shape[0]

    if query_seq_count == 0:
        print(">>> Error! The query lineage '" + query_seq_prefix + "'"
              + " is not in sequence data." + "\n")

        sys.exit()

    original_site_list = [int(x) for x in query_seq.columns.tolist()]

    sites_count = len(original_site_list)

    current_site_list = [site_idenx + i + 1 for i in range(sites_count)]

    columns_name_list = ["Original_Index", "Current_Index"] + lineage_name_list

    # sites_wic_data = pd.DataFrame(columns=columns_name_list,
    #                               index=current_site_list)

    sites_wic_data = pd.DataFrame(columns=columns_name_list)

    sites_wic_data["Original_Index"] = original_site_list

    sites_wic_data["Current_Index"] = current_site_list

    p = Pool(int(thread_num))

    lineage_wic_list = []

    for each_lineage in lineage_name_list:

        lineage_seq_df = seq_data[seq_data.index.str.contains(each_lineage) == True]

        lineage_result = p.apply_async(lineage_wic_run,
                                       args=(query_seq, each_lineage, lineage_seq_df,
                                             used_calEnt, query_seq_prefix))

        lineage_wic_list.append(lineage_result)


    p.close()
    p.join()


    for each_calculation in lineage_wic_list:
        lineage_wic = each_calculation.get()

        sites_wic_data[lineage_wic[0]] = lineage_wic[1]


    return sites_wic_data,sites_count




def wic_compute_engine(seq_file_path,
                       query_seq_prefix,
                       lineage_name_list,
                       parameter_dic,
                       site_dir):

    # start_memory = calculate_memory()

    block_size = parameter_dic["block_size"]

    data_blocks = 0

    read_mark = True

    sites_wic_pd_list = []

    gap_site_list = []

    invariant_site_list = []

    site_idenx = 0

    while read_mark:

        data_blocks += 1

        seq_pd,read_mark = piecewise_read(seq_file_path, block_size, data_blocks)
        
        # Data Preprocessing: Remove positions (columns) that consist entirely of “-” to adapt to scenarios where subsets are extracted from large alignment files for analysis in the pipeline.
        seq_pd = seq_pd.loc[:, ~(seq_pd == '-').all()]

        seq_pd_clean,gap_record = gap_processing(seq_pd, parameter_dic["gaps_use"])

        gap_site_list.append(gap_record)


        seq_pd_use,invariant_record = site_model(seq_pd_clean, parameter_dic["method"])

        invariant_site_list.append(invariant_record)


        if seq_pd_use.empty:
            print("    WIC for data_blocks {}".format(data_blocks) + " have been completed." + "\n")
            continue


        sites_wic_pd,last_site_count = wic_calculation(seq_pd_use,
                                       lineage_name_list,
                                       query_seq_prefix,
                                       parameter_dic,
                                       site_idenx)

        site_idenx += last_site_count


        sites_wic_pd_list.append(sites_wic_pd)


        print("    WIC for data_blocks {}".format(data_blocks) + " have been completed." + "\n")

    # print(sites_wic_pd_list)

    print(">>> " + "The WIC calculations of {} sites have been completed.".format(site_idenx)  + "\n")



    if not sites_wic_pd_list:
        print(">>> There are no variant sites in the data!")

        sys.exit()

    sites_probability_data = sites_wic_pd_list[0]


    for cookies_num in range(1,len(sites_wic_pd_list)):
        sites_probability_data = pd.concat([sites_probability_data,sites_wic_pd_list[cookies_num]],
                                           ignore_index= True)


#    site_ic_table = (site_dir + "/"
#                     + query_seq_prefix
#                     + "_site_WIC_from_lineages.xlsx")

    site_ic_csv = (site_dir + "/"+ query_seq_prefix + "_site_WIC.csv")

    # sites_probability_data.to_excel(excel_writer=site_ic_table,
    #                         index=False)

    sites_probability_data.to_csv(site_ic_csv, index=False, sep=",")


    # end_memory = calculate_memory()
    #
    # used_memory = end_memory - start_memory
    #
    #
    # print(">>> " + f"Occupied {used_memory}MB memory in the calculations of sites' WIC." + "\n")

    return sites_probability_data










# -*- coding: utf-8 -*-

"""
Virus recombination(VirusRecom), detecting recombination of viral lineages using information theory
Author: Zhou Zhi-Jian
Institution: Hunan University
Email: zjzhou@hnu.edu.cn
Copyright：Copyright (c) Zhi-Jian Zhou
Time: 2022/4/17 17:29

"""

import pandas as pd
import scipy.stats as stats

from my_func import (make_dir, mwic_calculation,
                     recomsplicing, wic_plot,
                     mwic_plot,recombreak_plot)

from wic_engine import wic_compute_engine



def virus_infor_calculate(seq_file_path,
                          query_seq_prefix,
                          lineage_list,
                          parameter_dic,
                          sub_outdir):

    is_use_gap = parameter_dic["gaps_use"]

    max_mic = parameter_dic["max_mic"]

    windows_size = parameter_dic["windows_size"]

    step_size = parameter_dic["step_size"]

    max_recom_fragment = parameter_dic["max_recom_fragment"]

    recom_percentage = parameter_dic["recom_percentage"]

    cumulative_major = parameter_dic["cumulative_major"]

    y_start = parameter_dic["y_start"]

    legend_location = parameter_dic["legend"]

    site_dir = sub_outdir + "/" + "WICs_of_sites"
    slide_window_dir = sub_outdir + "/" + "WICs_of_slide_window"
    run_record = sub_outdir + "/" + "run_record"

    make_dir(run_record)
    make_dir(site_dir)
    make_dir(slide_window_dir)


    identify_records_path = (run_record + "/"
                             + "identify_logs_detailed.txt")

    final_result_path = (sub_outdir + "/"
                         + "Possible_recombination_event_conciseness.txt")


    site_ic_csv_path = parameter_dic["input_wic"]


    if site_ic_csv_path == "":

        print(">>> Treat " + query_seq_prefix + " as a potential recombination lineage..." + "\n")

        print(">>> " + "VirusRecom starts calculating the weighted information content from each lineage..." + "\n")

        sites_probability_data = wic_compute_engine(seq_file_path,
                                              query_seq_prefix,
                                              lineage_list,
                                              parameter_dic,
                                              site_dir)

        if parameter_dic["only_wic"].upper() == "Y":

            print(">>> Finished, the output file is "
                  + "'" + site_ic_csv_path + "'"
                  + "\n")

            return


        else:
            pass


    else:

        sites_probability_data = pd.read_csv(site_ic_csv_path,
                                         sep=",",
                                         header=0)

    # print(sites_probability_data)

    lineage_name_list = list(sites_probability_data.columns[2:])

    # print(lineage_name_list)

    current_site_list = sites_probability_data["Current_Index"]

    original_site_list = sites_probability_data["Original_Index"]

    sites_count = len(current_site_list)

    site_map_dic = {}

    for n in range(len(current_site_list)):
        original_site_index = str(original_site_list[n])

        current_site_index = str(current_site_list[n])

        site_map_dic[current_site_index] = original_site_index


    if not parameter_dic["no_wic_figure"]:

        site_ic_fig = (site_dir + "/"
                       + query_seq_prefix
                       + "_site_WIC_from_lineages.pdf")

        wic_plot(lineage_name_list, original_site_list,
                 sites_probability_data, query_seq_prefix, site_ic_fig)


    mwic_out_table = (slide_window_dir + "/"
                      + query_seq_prefix
                      + "_mWIC_from_lineages.csv")

    window_ic_fig = (slide_window_dir + "/"
                     + query_seq_prefix
                     + "_mWIC_from_lineages.pdf")

    print(">>> " + "VirusRecom starts scanning using sliding window ..." + "\n")


    step_probability_data, slither_window_list = mwic_calculation(
        sites_probability_data,
        lineage_name_list,
        sites_count, site_map_dic,
        windows_size, step_size,
        mwic_out_table,
        parameter_dic["thread_num"])


    window_center_original = step_probability_data["Central_position(original)"]


    if not parameter_dic["no_mwic_figure"]:

        mwic_plot(is_use_gap,
                  lineage_name_list,
                  window_center_original,
                  step_probability_data,
                  query_seq_prefix,
                  window_ic_fig,
                  y_start,
                  legend_location)


    recombination_frag = {}


    for each_lineage in lineage_name_list:

        if not recombination_frag.__contains__(each_lineage):
            recombination_frag[each_lineage] = []

        potential_frag_list = []

        linegae_data_list = list(step_probability_data[
                                     each_lineage])

        for n in range(len(linegae_data_list)):

            line_ic_all = list(step_probability_data.iloc[n,2:])

            linegae_mwic = linegae_data_list[n]

            if (linegae_mwic == max(line_ic_all)
                    and linegae_mwic / max_mic >= recom_percentage):

                windows_center = slither_window_list[n][2]

                potential_frag_list.append(windows_center)

        recombination_frag[each_lineage] = potential_frag_list

    # print(recombination_frag)


    recom_region_dic = recomsplicing(sites_probability_data,
                                     sites_count,
                                     lineage_name_list,
                                     recombination_frag,
                                     step_size, max_mic,
                                     max_recom_fragment,
                                     recom_percentage)


    parents_region = {}

    for each_lineage in recom_region_dic:
        region_list = recom_region_dic[each_lineage]

        parents_region[each_lineage] = []

        range_list = []

        for each_region in region_list:
            each_region_s = each_region[0] + 1
            each_region_n = each_region[1] + 1

            region_range = int(site_map_dic[str(each_region_n)]) - int(
                site_map_dic[str(each_region_s)])

            range_list.append(region_range)

        parents_region[each_lineage].append(sum(range_list))

    # print(parents_region)

    major_parent = ""


    try:

        major_parent = max(parents_region, key=parents_region.get)

    except:
        print("    " + "No significant recombination events were found in "
              + query_seq_prefix + "\n")

        print("    " +
            "Note: Please check whether it is a false negative caused by a higher cp value given!"
              + "\n")


        with open(identify_records_path, "w",
                  encoding="utf-8") as identify_report_file:

            identify_report_file.write(
                "No significant recombination events were found in "

                + query_seq_prefix + "\n"+
                                     "Please check whether it is a false negative caused by a higher cp value given!")


        with open(final_result_path, "w",
                  encoding="utf-8") as final_result_file:

            final_result_file.write(
                "No significant recombination event were found in "
                + query_seq_prefix + "\n"
                + "Please check whether it is a false negative caused by a higher cp value given!")

        return

    finally:
        pass



    major_parent_ic = sum(list(sites_probability_data[major_parent]))

    mean_major_parent = major_parent_ic / sites_count

    if cumulative_major.upper() == "Y":

        accumulation_ic_list = []

        for lineage in lineage_name_list:
            accumulation_ic_list.append([lineage, sum(
                list(sites_probability_data[lineage])) / sites_count])

        hit_list_sort = sorted(accumulation_ic_list,
                               key=lambda x: x[1])

        max_ic_lineage = hit_list_sort[-1][0]

        if max_ic_lineage != major_parent:
            major_parent = max_ic_lineage


        major_parent_ic = sum(list(sites_probability_data[major_parent]))
        mean_major_parent = major_parent_ic / sites_count

    else:
        pass

    other_parental_markers = False


    recombination_dic = {}

    significant_recombination = {}

    for each_lineage in recom_region_dic:

        if each_lineage != major_parent:

            recombination_dic[each_lineage] = []
            significant_recombination[each_lineage] = []

            recom_region_list = recom_region_dic[each_lineage]

            for each_region in recom_region_list:
                each_region_start = each_region[0]
                each_region_end = each_region[1]

                this_lineage_ic_count = list(
                    sites_probability_data[each_lineage][
                    each_region_start:each_region_end])

                major_parent_ic_count = list(
                    sites_probability_data[major_parent][
                    each_region_start:each_region_end])

                region_mwic = sum(this_lineage_ic_count) / (
                        each_region_end - each_region_start + 1)

                left_start_site_original = site_map_dic[
                    str(each_region_start + 1)]

                right_end_site_original = site_map_dic[
                    str(min(each_region_end + 1, sites_count))]

                try:

                    zihe_test = stats.mannwhitneyu(this_lineage_ic_count,
                                                   major_parent_ic_count,
                                                   alternative="two-sided")

                    p_value = zihe_test[1]

                    recombination_dic[each_lineage].append([
                        left_start_site_original + " to " +
                        right_end_site_original + "(mWIC: " + str(
                            region_mwic) + ")", "p_value: " + str(p_value)])

                    if p_value < 0.05:
                        other_parental_markers = True

                        significant_recombination[each_lineage].append([
                            left_start_site_original + " to " +
                            right_end_site_original + "(mWIC: " + str(
                                region_mwic) + ")", "p_value: " + str(p_value)])


                except:

                    recombination_dic[each_lineage].append([
                        left_start_site_original + " to " +
                        right_end_site_original, "p_value: 1"])

                finally:
                    pass



    for i in list(significant_recombination.keys()):
        if not significant_recombination[i]:  #  == []
            del significant_recombination[i]

    if other_parental_markers:  #  == True

        print("\n" + "    "
              + "Possible major parent: " + major_parent
              + " (global mWIC: " + str(mean_major_parent) + ")" + "\n")

        print("    " + "Other possible parents and recombination region (map at the alignment): " + "\n")

        for each_lineage in significant_recombination:
            print("    " + each_lineage, significant_recombination[each_lineage])
            print("\n")

    else:
        print("    " + "No significant recombination events were found in "
              + query_seq_prefix + "\n")


    # print(recombination_dic)


    with open(identify_records_path, "w", encoding="utf-8") as identify_report_file:

        if not other_parental_markers:
            identify_report_file.write(
                "No significant recombination events were found in "

                + query_seq_prefix + "\n" * 2
                + "The most similar lineage: "
                + major_parent
                + "(global mWIC: " + str(mean_major_parent) + ")"
                + "\n" * 2
                + "The similar other lineage and not significant recombination regions (p>0.05):"
                + "\n")


        else:

            identify_report_file.write("Possible major parent: "
                                    + major_parent
                                    + "(global mWIC: "
                                    + str(mean_major_parent)
                                    + ")"
                                    + "\n" * 2
                                    + "Possible other parents and recombination regions:"
                                    + "\n")

        for key in recombination_dic:
            event_list = recombination_dic[key]

            identify_report_file.write(key + "\t")

            for each_event in event_list:
                identify_report_file.write(", ".join(each_event) + "\t")

            identify_report_file.write("\n")

        identify_report_file.write(
            "\n" + "Significance test of recombinant regions using Mann-Whitney-U test with two-tailed probabilities, "
                   "p-value less than 0.05 indicates a significant difference.")



    with open(final_result_path, "w", encoding="utf-8") as final_result_file:

        if not other_parental_markers:  #  == False
            final_result_file.write(
                "No significant recombination events were found in "
                + query_seq_prefix + "\n")

            final_result_file.write(
                "\n" + "Significance test of recombinant regions using Mann-Whitney-U test with two-tailed probabilities, "
                       "p-value less than 0.05 indicates a significant difference.")


        else:
            final_result_file.write("Possible major parent: "
                                  + major_parent
                                  + "(global mWIC: " + str(
                mean_major_parent) + ")"
                                  + "\n" * 2
                                  + "Other possible parents and significant recombination regions (p<0.05):"
                                  + "\n")

            for key in significant_recombination:
                event_list = significant_recombination[key]

                final_result_file.write(key + "\t")

                for each_envent in event_list:
                    final_result_file.write(", ".join(each_envent) + "\t")

                final_result_file.write("\n")

            final_result_file.write(
                "\n" + "Significance test of recombinant regions using Mann-Whitney-U test with two-tailed probabilities, "
                       "p-value less than 0.05 indicates a significant difference.")


    if parameter_dic["method"].upper() == "P" and parameter_dic["breakpoints"].upper() == "Y":
        print("\n" + "    "
              + "VirusRecom is running the algorithm of search for "
                     "recombination breakpoint..."
              + "\n")

        breakwins = parameter_dic["breakwins"]

        recombreak_plot(sites_probability_data,
                        lineage_name_list,
                        sites_count,
                        breakwins,
                        site_map_dic,
                        slide_window_dir,
                        query_seq_prefix,
                        parameter_dic["thread_num"])




# -*- coding: utf-8 -*-

"""
Author: Zhou Zhi-Jian
Institution: Hunan University
Email: zjzhou@hnu.edu.cn
Copyright：Copyright (c) Zhou Zhi-Jian
Time: 2022/5/20 20:18

"""

import subprocess


def SeqAlign(align_tool,thread_num,seq_file,out_file):

    print(
        "VirusRecom is running " + align_tool
        + " for sequence alignment..." + "\n")

    command_list = []

    command_list.append(align_tool)

    if align_tool.upper() == "MAFFT":

        command_list.append("--inputorder")
        command_list.append("--auto")
        command_list.append("--thread " + str(thread_num) + " ")
        command_list.append('"' + seq_file + '"' + " > "
                          + '"' + out_file + '"')


    elif align_tool.upper() == "MUSCLE":

        command_list.append("-maxiters 16")

        command_list.append(r" -in "
               + '"' + seq_file + '"'
               + " -out "
               + '"' + out_file + '"')


    elif align_tool.upper() == "CLUSTALO":

        command_list.append("--outfmt a2m")  # fasta format

        command_list.append("--threads " + str(thread_num))

        command_list.append("--force")

        command_list.append("--verbose")

        command_list.append(r" --in "
               + '"' + seq_file + '"'
               + " --out "
               + '"' + out_file + '"')


    align_command = " ".join(command_list)


    print(align_command)


    process = subprocess.Popen(align_command,
                               stdout=subprocess.PIPE,
                               stderr=subprocess.STDOUT,
                               universal_newlines=True,
                               shell=True)


    while True:

        output = process.stdout.readline()
        if process.poll() is not None:
            break

        elif output:
            print(output)

    process.terminate()

    return







# -*- coding: utf-8 -*-

"""
Author: Zhou Zhi-Jian
Institution: Hunan University
Email: zjzhou@hnu.edu.cn
Copyright：Copyright (c) Zhou Zhi-Jian
Time: 2023/3/1 17:01

"""
import sys
import os
sys.path.append(os.path.dirname(os.path.realpath(__file__)))



# -*- coding: utf-8 -*-

"""
Virus recombination(VirusRecom), detecting recombination of viral lineages using information theory
Author: Zhou Zhi-Jian
Institution: Hunan University
Email: zjzhou@hnu.edu.cn
Copyright：Copyright (c) Zhi-Jian Zhou
Time: 2022/4/17 17:29

"""

import sys
import os

app_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))

app_dir = app_dir.replace("\\", "/")

sys.path.append(app_dir)

import psutil

import numpy as np
import argparse

from datetime import datetime


from my_func import (handle_input_file,make_dir,
                     calEnt, calEnt_gap,
                     handle_exceptions,
                     others_analysis)


from corecp import virus_infor_calculate


example_use = r'''
----------------☆ Example of use ☆-----------------

  (1) If the input sequence-data has been aligned:
      virusrecom -a alignment.fasta -q query_name -l lineage_name_list.txt -g n -m p -w 100 -s 20 -o outdir
      
  (2) If the input sequence-data was not aligned: 
      virusrecom -ua unalignment.fasta -at mafft -q query_name -l lineage_name_list.txt -g n -m p -w 100 -s 20 -t 2 -o outdir
      
  (3) If you have a *csv file with site’s WIC value:
      virusrecom -iwic *csv -g n (or -g y) -q query_name -w 100 -s 20 -o outdir
  
  Tip: the input-file and output-directory is recommended absolute path.
  
  Above is just a conceptual example, detailed usage in website: https://github.com/ZhijianZhou01/virusrecom
  
----------------------☆  End  ☆---------------------

'''


def calculate_memory():
    pid = os.getpid()
    p = psutil.Process(pid)
    info = p.memory_full_info()
    memory = info.uss / 1024 / 1024
    return memory


def parameter():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        prog="virusrecom",
        description="",
        epilog=example_use)

    parser.add_argument(
        "-a", dest="alignment",
        help="Aligned sequence file (*.fasta). Note, each sequence name requires containing lineage mark.",
        default="")

    parser.add_argument(
        "-ua", dest="unalignment",
        help="Unaligned (non-alignment) sequence file (*.fasta). Note, each sequence name requires containing lineage mark.",
        default="")

    parser.add_argument(
        "-at", dest="align_tool",
        help="Program used for multiple sequence alignments (MSA). Supporting mafft, muscle and clustalo, such as ‘-at mafft’.",
        default="")

    parser.add_argument(
        "-iwic", dest="input_wic",
        help="Using the already obtained WIC values of reference lineages directly by a *.csv input-file.",
        default="")

    parser.add_argument(
        "-q", dest="query",
        help="Name of query lineage (usually potential recombinant), such as ‘-q xxxx’. Besides, ‘-q auto’ can scan all lineages as potential recombinant in turn.",
        default="")

    parser.add_argument(
        "-l", dest="lineages",
        help="Path of a text-file containing multiple lineage marks.",
        default="")

    parser.add_argument("-g", dest="gap",
                        help="Reserve sites containing gaps(-) in analyses? ‘-g y’ means to reserve, and ‘-g n’ means to delete.",
                        type=str,
                        default="")

    parser.add_argument("-m", dest="method",
                        help="Method for site scanning. ‘-m p’ uses polymorphic sites only, ‘-m a’ uses all the sites.",
                        type=str,
                        default="p")

    parser.add_argument("-w", dest="window",
                        help="Number of nucleotides sites per sliding window. Note: if the ‘-m p’ has been used, -w refers to the number of polymorphic sites per windows.",
                        type=int,
                        default=100)

    parser.add_argument("-s", dest="step",
                        help="Step size of the sliding window. Note: if the ‘-m p’ has been used, -s refers to the number of polymorphic sites per jump.",
                        type=int,
                        default=20)

    # max_recom_fragment
    parser.add_argument("-mr", dest="max_region",
                        help="The maximum allowed recombination region. Note: if the ‘-m p’ method has been used, it refers the maximum number of polymorphic sites contained in a recombinant region.",
                        type=int,
                        default=1000)

    parser.add_argument("-cp", dest="percentage",
                        help="The cutoff threshold of proportion (cp, default: 0.9) used for searching recombination regions when mWIC/EIC >= cp, the maximum value of cp is 1.",
                        type=float,
                        default=0.9)

    parser.add_argument("-cu", dest="cumulative",
                        help="Simply using the max cumulative WIC of all sites to identify the major parent. Off by default. If required, specify ‘-cu y’.",
                        type=str,
                        default="n")

    parser.add_argument("-b", dest="breakpoint",
                        help="Possible breakpoint scan of recombination. ‘-b y’ means yes, ‘-b n’ means no. Note: this option only takes effect when ‘-m p’ has been specified.",
                        type=str,
                        default="n")

    parser.add_argument("-bw", dest="breakwin",
                        help="The window size (default: 200) used for breakpoint scan, and step size is fixed at 1.",
                        type=int,
                        default=200)

    parser.add_argument(
        "-t", dest="thread",
        help="Number of threads (or cores) for calculations, default: 4.",
        type=int,
        default=4)

    parser.add_argument(
        "-y", dest="y_start",
        help="Starting value (default: 0) of the Y-axis in plot diagram.",
        type=float,
        default=0.0)

    parser.add_argument(
        "-le", dest="legend",
        help="The location of the legend, the default is adaptive. '-le r' indicates placed on the right.",
        default="auto")

    # upper right

    parser.add_argument(
        "-owic", dest="only_wic",
        help="Only calculate site WIC value. Off by default. If required, please specify ‘-owic y’.",
        type=str,
        default="n")

    parser.add_argument(
        "-e", dest="engrave",
        help="Write the file name to sequence names in batches. By specifying a directory containing one or multiple sequence files (*.fasta).",
        default="")

    parser.add_argument(
        "-en", dest="export_name",
        help="Export all sequence name of a *.fasta file.",
        default="")

    parser.add_argument(
        "-o", dest="outdir",
        help="Output directory to store all results.",
        type=str,
        default="")


    parser.add_argument(
        "--block",
        dest="block_size",
        help="Specifies the maximum number of sites per sub-block, different sub-blocks in sequence file will be sequentially loaded to calculate WIC. Default: 40000 sites.",
        type=int,
        default=40000)


    parser.add_argument(
        "--no_wic_fig",
        dest="no_wic_figure",
        action="store_true",
        help="Do not draw the image of WICs.")


    parser.add_argument(
        "--no_mwic_fig",
        dest="no_mwic_figure",
        action="store_true",
        help="Do not draw the image of mWICs.")


    myargs = parser.parse_args(sys.argv[1:])

    return myargs




def starts():

    print("\n" + "-------------------------------------------------")

    print("  Name: Virus Recombination (VirusRecom)")

    print(
        "  Description: Detecting recombination of viral lineages (or subtypes) using information theory.")

    print("  Version: 1.3.6 (2024-12-18)")

    print("  Author: Zhi-Jian Zhou")

    print("  Citation: Brief Bioinform. 2023 Jan 19;24(1)")

    print("-------------------------------------------------" + "\n")


    myargs = parameter()

    start = datetime.today().now()


    #  parameter

    parameter_dic = {}

    parameter_dic["aligned_seq"] = myargs.alignment  # path of the aligned sequence file

    parameter_dic["unaligned_seq"] = myargs.unalignment   # path of the unaligned sequence file

    parameter_dic["align_tool"] = myargs.align_tool  #  alignment tool used for no-aligned sequence file

    parameter_dic["input_wic"] = myargs.input_wic.replace("\\", "/")     # input_wic, if you have

    parameter_dic["query_lineage_name"] = myargs.query         #  the name of query lineage

    parameter_dic["lineage_file"] = myargs.lineages          # other lineages

    parameter_dic["gaps_use"] = myargs.gap.strip().upper()    #  strategy of handling gap

    parameter_dic["method"] = myargs.method.strip().upper()   #  Scan method

    parameter_dic["windows_size"] = myargs.window    #  window size

    parameter_dic["step_size"] = myargs.step     #  step size

    parameter_dic["max_recom_fragment"] = myargs.max_region  #  maximum recombination region

    parameter_dic["recom_percentage"] = myargs.percentage  #  the specific cutoff threshold of proportion

    parameter_dic["cumulative_major"] = myargs.cumulative

    parameter_dic["breakpoints"] =  myargs.breakpoint.strip()  #  whether to search for recombination breakpoints

    parameter_dic["breakwins"] = myargs.breakwin  #  Window size used to search for recombination breakpoints

    parameter_dic["thread_num"] = int(myargs.thread)  #  thread

    parameter_dic["y_start"] = myargs.y_start  #  Y-axis starting point when plotting

    parameter_dic["legend"] = myargs.legend

    parameter_dic["only_wic"] = myargs.only_wic.strip().upper()

    parameter_dic["engrave_name"] = myargs.engrave

    parameter_dic["export_name"] = myargs.export_name

    parameter_dic["out_dir"] = myargs.outdir.replace("\\", "/")  # output directory

    parameter_dic["no_wic_figure"] = myargs.no_wic_figure

    parameter_dic["no_mwic_figure"] = myargs.no_mwic_figure

    parameter_dic["block_size"] = int(myargs.block_size)


    print(myargs)

    print("\n")


    make_dir(parameter_dic["out_dir"])


    with open(parameter_dic["out_dir"] + "/" + "input-parameter.txt",
              "w", encoding="utf-8") as input_para:

        input_para.write(
            "the used parameter of virusrecom in command-line interface."
            + "\n" * 2)

        for key in list(parameter_dic.keys()):
            input_para.write(key + "\t" + str(parameter_dic[key]) + "\n")

    # update dic

    if parameter_dic["gaps_use"].upper() == "N":

        parameter_dic["calEnt_use"] = calEnt
        parameter_dic["max_mic"] = 2

    else:
        parameter_dic["calEnt_use"] = calEnt_gap

        parameter_dic["max_mic"] = np.log2(5)


    others = others_analysis(parameter_dic)

    if others:
        sys.exit()


    exceptions_label = handle_exceptions(parameter_dic)

    if exceptions_label:
        sys.exit()


    query_prefix = parameter_dic["query_lineage_name"]


    if parameter_dic["input_wic"] != "":

        virus_infor_calculate("",
                              query_prefix,
                              [],
                              parameter_dic,
                              parameter_dic["out_dir"])


    else:

        seq_alignment_file = handle_input_file(parameter_dic)

        lineage_mark_list = []

        with open(parameter_dic["lineage_file"]) as lineage_file:
            for line in lineage_file:
                line = line.strip()
                if line != "":
                    lineage_mark_list.append(line)


        if query_prefix.upper() != "AUTO":

            out_dir = parameter_dic["out_dir"]

            virus_infor_calculate(seq_alignment_file,
                                  query_prefix,
                                  lineage_mark_list,
                                  parameter_dic,
                                  out_dir)


        elif query_prefix.upper() == "AUTO":

            for each_lineage in lineage_mark_list:
                query_lineage_run = each_lineage

                new_lineage_name_list = []

                for lineage in lineage_mark_list:
                    if lineage != query_lineage_run:
                        new_lineage_name_list.append(lineage)

                sub_outdir = parameter_dic["out_dir"] + "/Results/" + query_lineage_run

                make_dir(sub_outdir)

                virus_infor_calculate(seq_alignment_file,
                                      query_lineage_run,
                                      new_lineage_name_list,
                                      parameter_dic,
                                      sub_outdir)



    duration = datetime.today().now() - start

    print(">>> " + "Take " + str(duration) + " seconds in total." + "\n")


    sys.exit()



if __name__ == "__main__":
    starts()


# -*- coding: utf-8 -*-

import setuptools

with open("README.md", "r",encoding="utf-8") as fh:
  long_description = fh.read()

setuptools.setup(
  name="virusrecom",
  version="1.3.6",
  author="Zhi-Jian Zhou",
  author_email="zjzhou@hnu.edu.cn",
  description="An information-theory-based method for recombination detection of viral lineages.",
  keywords="recombination, virus, evolution, information entropy",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/ZhijianZhou01/virusrecom",
  packages=setuptools.find_packages(),
  install_requires=["matplotlib>=2.2.5",
                    "numpy>=1.19.3",
                    "pandas>=1.1.5",
                    "scipy>=1.5.4",
                    "psutil>=5.9.1",
                    "openpyxl>=3.0.5"
                    ],

  classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Visualization",
        "Topic :: Scientific/Engineering :: Mathematics",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
        "Operating System :: OS Independent",
    ],
  entry_points={
             'console_scripts': [
                 'virusrecom = virusrecom.main:starts',
             ],
    }
)



# -*- coding: utf-8 -*-

"""
Author: Zhou Zhi-Jian
Institution: Hunan University
Email: zjzhou@hnu.edu.cn
Copyright：Copyright (c) Zhou Zhi-Jian
Time: 2022/5/20 20:09

"""


plt_corlor = [
"#fdb462",
"#b3de69",
"#fccde5",
"#d9d9d9",
"#e5d8bd",
"#ccebc5",
"#694d9f",
"#80b1d3",
"#bebada",
"#8dd3c7",
"#fb8072",
"olivedrab",
"darkorange",
"darkcyan",
"darkblue",
"dodgerblue",
"lightgreen",
"orangered",
"royalblue",
"indigo",
"forestgreen",
"lightsteelblue",
"burlywood",
"antiquewhite",
"limegreen",
"darkgreen",
"darkmagenta",
"cornflowerblue",
"grey",
"tan",
"slategrey",
"ghostwhite",
"navajowhite",
"lavender",
"darkgrey",
"blanchedalmond",
"lime",
"midnightblue",
"darkgray",
"papayawhip",
"seagreen",
"navy",
"silver",
"moccasin",
"mediumseagreen",
"lightgray",
"springgreen",
"mediumblue",
"lightgrey",
"wheat",
"mintcream",
"blue",
"gainsboro",
"oldlace",
"mediumspringgreen",
"whitesmoke",
"floralwhite",
"mediumaquamarine",
"slateblue",
"darkgoldenrod",
"aquamarine",
"darkslateblue",
"goldenrod",
"turquoise",
"mediumslateblue",
"snow",
"cornsilk",
"lightseagreen",
"mediumpurple",
"lightcoral",
"rosybrown",
"gold",
"lemonchiffon",
"azure",
"mediumturquoise",
"blueviolet",
"indianred",
"khaki",
"lightcyan",
"darkorchid",
"brown",
"palegoldenrod",
"paleturquoise",
"darkviolet",
"darkkhaki",
"darkslategray",
"mediumorchid",
"ivory",
"darkslategrey",
"thistle",
"darkred",
"beige",
"teal",
"plum",
"lightyellow",
"lightgoldenrodyellow",
"purple",
"mistyrose",
"olive",
"cyan",
"salmon",
"aqua",
"m",
"tomato",
"yellow",
"darkturquoise",
"fuchsia",
"darksalmon",
"cadetblue",
"magenta",
"coral",
"yellowgreen",
"powderblue",
"orchid",
"darkolivegreen",
"lightblue",
"mediumvioletred",
"lightsalmon",
"greenyellow",
"deepskyblue",
"deeppink",
"chartreuse",
"skyblue",
"hotpink",
"seashell",
"lawngreen",
"lightskyblue",
"lavenderblush",
"sage",
"steelblue",
"palevioletred",
"saddlebrown",
"lightsage",
"aliceblue",
"crimson",
"pink",
"sandybrown",
"peachpuff",
"honeydew",
"lightslategrey",
"lightpink",
"peru",
"darkseagreen",
"lightslategray",
"linen",
"palegreen",
"slategray",
"dimgray",
"dimgrey",
"bisque",
"chocolate",
"#1F77B4",
"firebrick",
"green",
"orange",
"#9467BD",
"sienna",
"violet",
"gray",
"y",
"#17BECF",
"#8dd3c7",
]